#!/usr/bin/python

import math
import debug

def buildSCM(example_set, penalty_value, stopping_point, feature_set):

	# DEBUG
	my_debug_object = debug.debug_object()
	
	# initialization
	result_set = []
	Q={}
	R={}
	positive_examples_set = []
	negative_examples_set = []
	iterations = 0
	
	# 1. Seperate positive and negative examples
	#    (first element of an example must be 1 or 0 to indicate positive or negative)
	positive_examples_set = \
		filter(lambda example : example[0] == 1, example_set) # ie P
	negative_examples_set = \
		filter(lambda example : example[0] == 0, example_set) # ie N
	
	if debug.DEBUG:
		print
		print "-----------------------------------------------------------"
		print "-----------------------------------------------------------"	
		print "-----------------------------------------------------------"
		print "example set"
		print example_set
		print "-----------------------------------------------------------"
		print "BALLS(FEATURES):"
		for feature_id in range(len(feature_set)):
			print repr(feature_id) + ":",
			feature_set[feature_id].print_specs()
		print "-----------------------------------------------------------"
		print "positive_examples_set:"
		print positive_examples_set
		print "negative_examples_set:"
		print negative_examples_set
		print
		my_debug_object.debug_write_initial_set(positive_examples_set,negative_examples_set)
		
	# 2. create coverage sets
	for feature_id in range(len(feature_set)):
		# get the feature we're looking at (h_i)
		feature = feature_set[feature_id]
				
		# determine the subset of negative_examples_set (N) 
		# covered by our feature (h_i)
		Q[feature_id] = \
			filter( lambda (is_positive, data_point) : feature.covers_neg_point(data_point) , \
				negative_examples_set)
		
		# determine the subset of positive_examples_set (P) on 
		# which our feature (h_i) makes a mistake
		R[feature_id] = \
			filter( lambda (is_positive, data_point) : feature.errs_pos_point(data_point) , \
				positive_examples_set)
	
	# Repeat until done (step 6. from text)
	while (len(negative_examples_set)>0) and (len(result_set)<stopping_point):
	
		iterations = iterations+1
		
		if debug.DEBUG:
			print "-----------------------------------------------------------"
			print "Beginning new iteration of loop"
			print 
			
		# 3. Find h_k, the feature with the largest usefulness value ( |Q_k|-p*|R_k| )
		largest_usefulness_value = -999999 #todo: we need to find a minus infinity or something like that
		largest_usefulness_feature_id = -1 
		for feature_id in range(len(feature_set)):
			usefulness_value = len(Q[feature_id]) - (penalty_value * len(R[feature_id]))
			if debug.DEBUG:
				print "\t%d scores %s" % (feature_id, usefulness_value)			
			if usefulness_value > largest_usefulness_value:
				largest_usefulness_value = usefulness_value
				largest_usefulness_feature_id = feature_id
		# validity checking
		if largest_usefulness_feature_id==-1:
			raise ("Could not find feature with maximum usefulness value. largest_usefulness_value in loop may be too small.")
			
		if debug.DEBUG:	
			print "largest_usefulness_value, largest_usefulness_feature_id"
			print largest_usefulness_value, largest_usefulness_feature_id
			print
			my_debug_object.debug_write_ball( feature_set[largest_usefulness_feature_id], Q[largest_usefulness_feature_id], R[largest_usefulness_feature_id])

		# 4.a. Remember our "winning" feature
		
		result_set.append( feature_set[largest_usefulness_feature_id] )
				
		if debug.DEBUG:
			print "result_set"
			for ball in result_set:
				ball.print_specs()
			print
			print "winning Q"
			print Q[largest_usefulness_feature_id]
			print "winning R"
			print R[largest_usefulness_feature_id]

		# 4.b. Reduce negative_examples_set and positive_examples_set
		
		negative_examples_set = filter ( \
			lambda datapoint : datapoint not in Q[largest_usefulness_feature_id], \
			negative_examples_set )
		positive_examples_set = filter ( \
			lambda datapoint : datapoint not in R[largest_usefulness_feature_id], \
			positive_examples_set )

		# 5. remove from Q and R those datapoints that were covered by Q_k and R_k.
		
		# make copies of the most useful Q & R's 
		most_useful_Q = Q[largest_usefulness_feature_id]
		most_useful_R = R[largest_usefulness_feature_id]
		for feature_id in range(len(feature_set)):
			#if debug.DEBUG:		
			#	print feature_id
			#	print "\tQ before:" + repr(Q[feature_id])
			Q[feature_id] = filter ( \
				lambda datapoint : datapoint not in most_useful_Q,
				Q[feature_id] )
			R[feature_id] = filter ( \
				lambda datapoint : datapoint not in most_useful_R,
				R[feature_id] )
			#if debug.DEBUG:	
			#	print "\tQ after :" + repr(Q[feature_id])
			#	print

		if debug.DEBUG:
			print "----reduced set:"
			print "positive_examples_set:"
			print positive_examples_set
			print "negative_examples_set:"
			print negative_examples_set
			print


	# DEBUG
	my_debug_object.close()

	# 7. We're done. Create f(x) combining our result set
	return result_set
	
