class dataDependentBall:

	# "create" the ball
	def __init__ (self, is_positive, center_point, radius, distance_function):
		self.is_positive = is_positive
		self.center_point = center_point
		self.radius = radius
		self.distance_function = distance_function
	
	# determine if ball covers another data point
	def covers(self, datapoint):
		if (self.distance_function(self.center_point, datapoint) <= self.radius):
			return bool(self.is_positive)
		else:
			return not bool(self.is_positive)
			
	def covers_neg_point(self, datapoint):
		# by defn, the datapoint must be a negative datapoint (it should come from N)
		if self.is_positive:
			# positive ball: get a score for a datapoint outside the ball
			if (self.distance_function(self.center_point, datapoint) <= self.radius):
				return 0
			else:
				return 1
		else:	
			# negative ball: get a "score" for datapoint within the ball
			if (self.distance_function(self.center_point, datapoint) <= self.radius):
				return 1
			else:
				return 0

	def errs_pos_point(self, datapoint):
		# by defn, the datapoint must be a positive datapoint (it should come from N)
		if self.is_positive:
			# positive ball: count an error for a datapoint outside the ball
			if (self.distance_function(self.center_point, datapoint) <= self.radius):
				return 0
			else:
				return 1
		else:	
			# negative ball: count an error for datapoint within the ball
			if (self.distance_function(self.center_point, datapoint) <= self.radius):
				return 1
			else:
				return 0
				
	# print out specs for the ball
	def print_specs(self):
		print "ball: " + repr([self.is_positive, self.center_point, self.radius])
		
	def specs(self):
		return "ball: " + repr([self.is_positive, self.center_point, self.radius])
		
	def xml(self):
		return ('\t\t<ball is_positive="%d" center_x="%d" center_y="%d" radius="%d"/>\n' % (self.is_positive, self.center_point[0], self.center_point[1], self.radius))
		
# make a ball from each data point to every POSITIVE data point (see page 731, 2nd to last para)
# data should be in the form 
# (0,1) X Y Z ...
def makeBalls(training_data, distance_function, epsilon = 1):

	balls = []
	
	# loop through center points
	for (is_positive, center_datapoint ) in training_data:
			
		# loop through edge points
		for (edge_is_positive, edge_datapoint) in training_data:
		
			if (not edge_is_positive):
				continue # skip over negative edge points (pg 731 "the ball h of largest usefullness value U_h always has a radius such that a P example is located at its border."
				
			radius = distance_function(center_datapoint, edge_datapoint)
							
			#if (radius==0):
			#	continue # skip over 0-radius balls (TODO: this could be optimized)
				
			if (is_positive):	
				new_ball = dataDependentBall(is_positive, center_datapoint, radius+epsilon, distance_function)
			else:
				new_ball = dataDependentBall(is_positive, center_datapoint, radius-epsilon, distance_function)
			balls.append ( new_ball )
			
	return balls
